@@ -1,7 +1,9 @@
+from glob import glob
 from os import makedirs
+from os.path import basename
 from os.path import isdir
-from os.path import isfile
 from os.path import join as path_join
+from shutil import copy
 from shutil import move
 from shutil import rmtree
 from sqlite3 import OperationalError
@@ -94,7 +96,61 @@ def make_database_3(db: Connection):
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["LASTSTART", "0"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["COOKIES", "{}"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["FILESFOLDER", "FA.files"], False)
-    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["VERSION", str(__database_version__)], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["VERSION", "3.0.0"], False)
+
+    db.commit()
+
+
+def make_database_3_1(db: Connection):
+    # Create submissions table
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS SUBMISSIONS
+        (ID INT UNIQUE NOT NULL,
+        AUTHOR TEXT NOT NULL,
+        TITLE TEXT,
+        UDATE DATE NOT NULL,
+        DESCRIPTION TEXT,
+        TAGS TEXT,
+        CATEGORY TEXT,
+        SPECIES TEXT,
+        GENDER TEXT,
+        RATING TEXT,
+        FILELINK TEXT,
+        FILEEXT TEXT,
+        FILESAVED INT,
+        PRIMARY KEY (ID ASC));"""
+    )
+
+    # Create users table
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS USERS
+        (USERNAME TEXT UNIQUE NOT NULL,
+        FOLDERS TEXT NOT NULL,
+        GALLERY TEXT,
+        SCRAPS TEXT,
+        FAVORITES TEXT,
+        MENTIONS TEXT,
+        PRIMARY KEY (USERNAME ASC));"""
+    )
+
+    # Create settings table
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS SETTINGS
+        (SETTING TEXT UNIQUE,
+        SVALUE TEXT,
+        PRIMARY KEY (SETTING ASC));"""
+    )
+
+    db.commit()
+
+    # Add settings
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["USRN", "0"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["SUBN", "0"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["LASTUPDATE", "0"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["LASTSTART", "0"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["COOKIES", "{}"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["FILESFOLDER", "FA.files"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["VERSION", "3.1.0"], False)
 
     db.commit()
 
@@ -136,16 +192,19 @@ def update_2_7_to_3(db: Connection) -> Connection:
             print(user_n, end="\r", flush=True)
             folders_new: List[str] = []
             for folder in folders.split(","):
-                if folder == "g":
-                    folders_new.append("gallery")
+                folder_new = ""
+                folder_disabled = folder.endswith("!")
+                if (folder := folder.strip("!")) == "g":
+                    folder_new = "gallery"
                 elif folder == "s":
-                    folders_new.append("scraps")
+                    folder_new = "scraps"
                 elif folder == "f":
-                    folders_new.append("favorites")
+                    folder_new = "favorites"
                 elif folder == "e":
-                    folders_new.append("extras")
+                    folder_new = "extras"
                 elif folder == "E":
-                    folders_new.append("Extras")
+                    folder_new = "Extras"
+                folders_new.append(folder_new + ("!" if folder_disabled else ""))
             update(db_new, "USERS", ["FOLDERS"], [",".join(folders_new)], "USERNAME", user)
             db_new.commit() if user_n % 1000 == 0 else None
         db_new.commit()
@@ -155,20 +214,26 @@ def update_2_7_to_3(db: Connection) -> Connection:
         print("Update submissions FILEEXT and FILESAVED and move to new location")
         sub_n: int = 0
         sub_not_found: List[int] = []
-        for id_, location, filename in select_all(db, "SUBMISSIONS", ["ID", "LOCATION", "FILENAME"]):
+        for id_, location in select_all(db, "SUBMISSIONS", ["ID", "LOCATION"]):
             sub_n += 1
             print(sub_n, end="\r", flush=True)
-            sub_folder: str = path_join("FA.files", location.strip("/"))
-            sub_folder_new: str = path_join("FA.files_new", tiered_path(id_))
-            if isfile(sub_file := path_join(sub_folder, filename)):
-                makedirs(sub_folder_new, exist_ok=True)
-                move(sub_file, path_join(sub_folder_new, filename))
-                update(db_new, "SUBMISSIONS", ["FILEEXT", "FILESAVED"], [filename.split(".")[-1], True], "ID", id_)
-            elif isfile(path_join(sub_folder_new, filename)):
-                update(db_new, "SUBMISSIONS", ["FILEEXT", "FILESAVED"], [filename.split(".")[-1], True], "ID", id_)
+            if isdir(folder_old := path_join("FA.files", location.strip("/"))):
+                fileglob = glob(path_join(folder_old, "submission*"))
+                fileext = ""
+                if filename := fileglob[0] if fileglob else "":
+                    makedirs((folder_new := path_join("FA.files_new", tiered_path(id_))), exist_ok=True)
+                    copy(filename, path_join(folder_new, (filename := basename(filename))))
+                    fileext = filename.split(".")[-1] if "." in filename else ""
+                else:
+                    sub_not_found.append(id_)
+                update(db_new, "SUBMISSIONS", ["FILEEXT", "FILESAVED"], [fileext, bool(filename)], "ID", id_)
+                rmtree(folder_old)
+            elif isdir(path_join("FA.files_new", tiered_path(id_))):
+                # if this block is reached, original folder was removed and database entry updated
+                continue
             else:
                 sub_not_found.append(id_)
-                update(db_new, "SUBMISSIONS", ["FILEEXT", "FILESAVED"], [filename.split(".")[-1], False], "ID", id_)
+                update(db_new, "SUBMISSIONS", ["FILEEXT", "FILESAVED"], ["", False], "ID", id_)
             db_new.commit() if sub_n % 10000 == 0 else None
         db_new.commit()
         print()
@@ -221,7 +286,7 @@ def update_3_to_3_1(db: Connection) -> Connection:
 
     try:
         db_new = connect_database("FA_new.db")
-        make_database(db_new)
+        make_database_3_1(db_new)
 
         # Transfer common submissions and users data
         print("Transfer common submissions and users data")
@@ -238,6 +303,7 @@ def update_3_to_3_1(db: Connection) -> Connection:
             """INSERT OR REPLACE INTO db_new.SETTINGS
             SELECT * FROM SETTINGS"""
         )
+        db.execute("UPDATE db_new.SETTINGS SET SVALUE = '3.1.0' WHERE SETTING = 'VERSION'")
 
         db.commit()
         db_new.commit()
@@ -267,6 +333,55 @@ def update_3_to_3_1(db: Connection) -> Connection:
     return connect_database("FA.db")
 
 
+def update_3_1_to_3_2(db: Connection) -> Connection:
+    print("Updating 3.1.0 to 3.2.0")
+    db_new: Optional[Connection] = None
+
+    try:
+        db_new = connect_database("FA_new.db")
+        make_database(db_new)
+
+        # Transfer common submissions and users data
+        print("Transfer common submissions and users data")
+        db.execute("ATTACH DATABASE 'FA_new.db' AS db_new")
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.SUBMISSIONS
+            SELECT * FROM SUBMISSIONS"""
+        )
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.USERS(USERNAME, FOLDERS, GALLERY, SCRAPS, FAVORITES, MENTIONS)
+        SELECT USERNAME, FOLDERS, GALLERY, SCRAPS, FAVORITES, MENTIONS FROM USERS"""
+        )
+        db.execute(
+            """INSERT OR REPLACE INTO db_new.SETTINGS
+            SELECT * FROM SETTINGS"""
+        )
+        db.execute("UPDATE db_new.SETTINGS SET SVALUE = '3.2.0' WHERE SETTING = 'VERSION'")
+        db.execute("UPDATE db_new.USERS SET JOURNALS = ''")
+
+        db.commit()
+        db_new.commit()
+
+        # Close databases and replace old database
+        print("Close databases and replace old database")
+        db.commit()
+        db.close()
+        db_new.commit()
+        db_new.close()
+        move("FA.db", "FA_3.db")
+        move("FA_new.db", "FA.db")
+    except (BaseException, Exception) as err:
+        print("Database update interrupted!")
+        db.commit()
+        db.close()
+        if db_new is not None:
+            db_new.commit()
+            db_new.close()
+        raise err
+
+    return connect_database("FA.db")
+
+
 def update_database(db: Connection) -> Connection:
     if not (db_version := get_version(db)):
         raise Exception("Cannot read version from database.")
@@ -277,8 +392,12 @@ def update_database(db: Connection) -> Connection:
     elif compare_versions(db_version, "2.7.0") == 0:
         db = update_2_7_to_3(db)
         db = update_3_to_3_1(db)
+        db = update_3_1_to_3_2(db)
     elif compare_versions(db_version, "3.0.0") == 0:
         db = update_3_to_3_1(db)
+        db = update_3_1_to_3_2(db)
+    elif compare_versions(db_version, "3.1.0") == 0:
+        db = update_3_1_to_3_2(db)
     elif compare_versions(db_version, "2.7.0") < 0:
         raise Exception("Update does not support versions lower than 2.11.2")
 
